@@ -1,5 +1,7 @@
 from __future__ import annotations
 
+from typing_extensions import override
+
 from pyinfra.api import FactBase
 
 from .util.packaging import parse_packages
@@ -18,6 +20,7 @@ class ChocoPackages(FactBase):
         }
     """
 
+    @override
     def command(self) -> str:
         return "choco list"
 
@@ -25,6 +28,7 @@ class ChocoPackages(FactBase):
 
     default = dict
 
+    @override
     def process(self, output):
         return parse_packages(CHOCO_REGEX, output)
 
@@ -34,8 +38,10 @@ class ChocoVersion(FactBase):
     Returns the choco (Chocolatey) version.
     """
 
+    @override
     def command(self) -> str:
         return "choco --version"
 
+    @override
     def process(self, output):
         return "".join(output).replace("\n", "")
