@@ -5,7 +5,6 @@ Linux/BSD.
 
 from __future__ import annotations
 
-import shlex
 from io import StringIO
 from itertools import filterfalse, tee
 from os import path
@@ -18,10 +17,10 @@ from pyinfra.api.util import try_int
 from pyinfra.connectors.util import remove_any_sudo_askpass_file
 from pyinfra.facts.files import Directory, FindInFile, Link
 from pyinfra.facts.server import (
-    Crontab,
     Groups,
     Home,
     Hostname,
+    Kernel,
     KernelModules,
     Locales,
     Mounts,
@@ -30,6 +29,7 @@ from pyinfra.facts.server import (
     Users,
     Which,
 )
+from pyinfra.operations import crontab as crontab_
 
 from . import (
     apk,
@@ -49,7 +49,7 @@ from . import (
     yum,
     zypper,
 )
-from .util.files import chmod, sed_replace
+from .util.files import chmod
 
 if TYPE_CHECKING:
     from pyinfra.api.arguments_typed import PyinfraOperation
@@ -89,7 +89,7 @@ def reboot(delay=10, interval=1, reboot_timeout=300):
         sleep(delay)
         max_retries = round(reboot_timeout / interval)
 
-        host.connection = None  # remove the connection object
+        host.disconnect()  # make sure we are properly disconnected
         retries = 0
 
         while True:
@@ -336,7 +336,15 @@ def mount(
         mounted_options = mounts[path]["options"]
         needed_options = set(options) - set(mounted_options)
         if needed_options:
-            yield "mount -o remount,{0} {1}".format(options_string, path)
+            if host.get_fact(Kernel).strip() == "FreeBSD":
+                fs_type = mounts[path]["type"]
+                device = mounts[path]["device"]
+
+                yield "mount -o update,{options} -t {fs_type} {device} {path}".format(
+                    options=options_string, fs_type=fs_type, device=device, path=path
+                )
+            else:
+                yield "mount -o remount,{0} {1}".format(options_string, path)
 
     else:
         host.noop(
@@ -438,7 +446,7 @@ def sysctl(
     existing_sysctls = host.get_fact(Sysctl, keys=[key])
     existing_value = existing_sysctls.get(key)
 
-    if not existing_value or existing_value != value:
+    if existing_value != value:
         yield "sysctl {0}='{1}'".format(key, string_value)
     else:
         host.noop("sysctl {0} is set to {1}".format(key, string_value))
@@ -510,7 +518,7 @@ def service(
 
     else:
         raise OperationError(
-            ("No init system found " "(no systemctl, initctl, /etc/init.d or /etc/rc.d found)"),
+            ("No init system found (no systemctl, initctl, /etc/init.d or /etc/rc.d found)"),
         )
 
     yield from service_operation._inner(
@@ -588,180 +596,7 @@ def packages(
     yield from package_operation._inner(packages=packages, present=present)
 
 
-@operation()
-def crontab(
-    command: str,
-    present=True,
-    user: str | None = None,
-    cron_name: str | None = None,
-    minute="*",
-    hour="*",
-    month="*",
-    day_of_week="*",
-    day_of_month="*",
-    special_time: str | None = None,
-    interpolate_variables=False,
-):
-    """
-    Add/remove/update crontab entries.
-
-    + command: the command for the cron
-    + present: whether this cron command should exist
-    + user: the user whose crontab to manage
-    + cron_name: name the cronjob so future changes to the command will overwrite
-    + minute: which minutes to execute the cron
-    + hour: which hours to execute the cron
-    + month: which months to execute the cron
-    + day_of_week: which day of the week to execute the cron
-    + day_of_month: which day of the month to execute the cron
-    + special_time: cron "nickname" time (@reboot, @daily, etc), overrides others
-    + interpolate_variables: whether to interpolate variables in ``command``
-
-    Cron commands:
-        Unless ``name`` is specified the command is used to identify crontab entries.
-        This means commands must be unique within a given users crontab. If you require
-        multiple identical commands, provide a different name argument for each.
-
-    Special times:
-        When provided, ``special_time`` will be used instead of any values passed in
-        for ``minute``/``hour``/``month``/``day_of_week``/``day_of_month``.
-
-    **Example:**
-
-    .. code:: python
-
-        # simple example for a crontab
-        server.crontab(
-            name="Backup /etc weekly",
-            command="/bin/tar cf /tmp/etc_bup.tar /etc",
-            name="backup_etc",
-            day_of_week=0,
-            hour=1,
-            minute=0,
-        )
-    """
-
-    def comma_sep(value):
-        if isinstance(value, (list, tuple)):
-            return ",".join("{0}".format(v) for v in value)
-        return value
-
-    minute = comma_sep(minute)
-    hour = comma_sep(hour)
-    month = comma_sep(month)
-    day_of_week = comma_sep(day_of_week)
-    day_of_month = comma_sep(day_of_month)
-
-    crontab = host.get_fact(Crontab, user=user)
-    name_comment = "# pyinfra-name={0}".format(cron_name)
-
-    existing_crontab = crontab.get(command)
-    existing_crontab_command = command
-    existing_crontab_match = command
-
-    if not existing_crontab and cron_name:  # find the crontab by name if provided
-        for cmd, details in crontab.items():
-            if not details["comments"]:
-                continue
-            if name_comment in details["comments"]:
-                existing_crontab = details
-                existing_crontab_match = cmd
-                existing_crontab_command = cmd
-
-    exists = existing_crontab is not None
-
-    edit_commands: list[str | StringCommand] = []
-    temp_filename = host.get_temp_filename()
-
-    if special_time:
-        new_crontab_line = "{0} {1}".format(special_time, command)
-    else:
-        new_crontab_line = "{minute} {hour} {day_of_month} {month} {day_of_week} {command}".format(
-            minute=minute,
-            hour=hour,
-            day_of_month=day_of_month,
-            month=month,
-            day_of_week=day_of_week,
-            command=command,
-        )
-
-    existing_crontab_match = ".*{0}.*".format(existing_crontab_match)
-
-    # Don't want the cron and it does exist? Remove the line
-    if not present and exists:
-        edit_commands.append(
-            sed_replace(
-                temp_filename,
-                existing_crontab_match,
-                "",
-                interpolate_variables=interpolate_variables,
-            ),
-        )
-
-    # Want the cron but it doesn't exist? Append the line
-    elif present and not exists:
-        if cron_name:
-            if crontab:  # append a blank line if cron entries already exist
-                edit_commands.append("echo '' >> {0}".format(temp_filename))
-            edit_commands.append(
-                "echo {0} >> {1}".format(
-                    shlex.quote(name_comment),
-                    temp_filename,
-                ),
-            )
-
-        edit_commands.append(
-            "echo {0} >> {1}".format(
-                shlex.quote(new_crontab_line),
-                temp_filename,
-            ),
-        )
-
-    # We have the cron and it exists, do it's details? If not, replace the line
-    elif present and exists:
-        assert existing_crontab is not None
-        if any(
-            (
-                special_time != existing_crontab.get("special_time"),
-                try_int(minute) != existing_crontab.get("minute"),
-                try_int(hour) != existing_crontab.get("hour"),
-                try_int(month) != existing_crontab.get("month"),
-                try_int(day_of_week) != existing_crontab.get("day_of_week"),
-                try_int(day_of_month) != existing_crontab.get("day_of_month"),
-                existing_crontab_command != command,
-            ),
-        ):
-            edit_commands.append(
-                sed_replace(
-                    temp_filename,
-                    existing_crontab_match,
-                    new_crontab_line,
-                    interpolate_variables=interpolate_variables,
-                ),
-            )
-
-    if edit_commands:
-        crontab_args = []
-        if user:
-            crontab_args.append("-u {0}".format(user))
-
-        # List the crontab into a temporary file if it exists
-        if crontab:
-            yield "crontab -l {0} > {1}".format(" ".join(crontab_args), temp_filename)
-
-        # Now yield any edits
-        for edit_command in edit_commands:
-            yield edit_command
-
-        # Finally, use the tempfile to write a new crontab
-        yield "crontab {0} {1}".format(" ".join(crontab_args), temp_filename)
-    else:
-        host.noop(
-            "crontab {0} {1}".format(
-                command,
-                "exists" if present else "does not exist",
-            ),
-        )
+crontab = crontab_.crontab
 
 
 @operation()
@@ -938,6 +773,7 @@ def user(
     shell: str | None = None,
     group: str | None = None,
     groups: list[str] | None = None,
+    append=False,
     public_keys: str | list[str] | None = None,
     delete_keys=False,
     ensure_home=True,
@@ -945,7 +781,6 @@ def user(
     system=False,
     uid: int | None = None,
     comment: str | None = None,
-    add_deploy_dir=True,
     unique=True,
     password: str | None = None,
 ):
@@ -958,14 +793,14 @@ def user(
     + shell: the users shell
     + group: the users primary group
     + groups: the users secondary groups
+    + append: whether to add `user` to `groups`, w/o losing membership of other groups
     + public_keys: list of public keys to attach to this user, ``home`` must be specified
     + delete_keys: whether to remove any keys not specified in ``public_keys``
     + ensure_home: whether to ensure the ``home`` directory exists
-    + create_home: whether to new user create home directories from the system skeleton
+    + create_home: whether user create new user home directories from the system skeleton
     + system: whether to create a system account
     + uid: use a specific userid number
     + comment: the user GECOS comment
-    + add_deploy_dir: any public_key filenames are relative to the deploy directory
     + unique: prevent creating users with duplicate UID
     + password: set the encrypted password for the user
 
@@ -1064,22 +899,29 @@ def user(
 
         if create_home:
             args.append("-m")
-        else:
+        elif os_type != "FreeBSD":
             args.append("-M")
 
-        if password:
+        if password and os_type != "FreeBSD":
             args.append("-p '{0}'".format(password))
 
         # Users are often added by other operations (package installs), so check
         # for the user at runtime before adding.
         add_user_command = "useradd"
+
         if os_type == "FreeBSD":
             add_user_command = "pw useradd"
-            yield "{0} -n {2} {1}".format(
-                add_user_command,
-                " ".join(args),
-                user,
-            )
+
+            if password:
+                yield "echo '{3}' | {0} -n {2} -H 0 {1}".format(
+                    add_user_command, " ".join(args), user, password
+                )
+            else:
+                yield "{0} -n {2} {1}".format(
+                    add_user_command,
+                    " ".join(args),
+                    user,
+                )
         else:
             yield "{0} {1} {2}".format(
                 add_user_command,
@@ -1104,14 +946,22 @@ def user(
             args.append("-g {0}".format(group))
 
         # Check secondary groups, if defined
-        if groups and set(existing_user["groups"]) != set(groups):
-            args.append("-G {0}".format(",".join(groups)))
+        if groups:
+            if append:
+                if not set(groups).issubset(existing_user["groups"]):
+                    args.append("-a")
+                    args.append("-G {0}".format(",".join(groups)))
+            elif set(existing_user["groups"]) != set(groups):
+                args.append("-G {0}".format(",".join(groups)))
 
         if comment and existing_user["comment"] != comment:
             args.append("-c '{0}'".format(comment))
 
         if password and existing_user["password"] != password:
-            args.append("-p '{0}'".format(password))
+            if os_type == "FreeBSD":
+                yield "echo '{0}' | pw usermod -n {1} -H 0".format(password, user)
+            else:
+                args.append("-p '{0}'".format(password))
 
         # Need to mod the user?
         if args:
@@ -1119,18 +969,6 @@ def user(
                 yield "pw usermod -n {1} {0}".format(" ".join(args), user)
             else:
                 yield "usermod {0} {1}".format(" ".join(args), user)
-            if comment:
-                existing_user["comment"] = comment
-            if home:
-                existing_user["home"] = home
-            if shell:
-                existing_user["shell"] = shell
-            if group:
-                existing_user["group"] = group
-            if groups:
-                existing_user["groups"] = groups
-            if password:
-                existing_user["password"] = password
 
     # Ensure home directory ownership
     if ensure_home and home:
